@@ -2,8 +2,8 @@
 # coding: utf-8
 # Copyright (c) Modos Team, 2020
 
-import re
 import json
+import re
 from typing import Dict, Any, Optional, Tuple, List, Union, Callable
 
 from dateparser import parse
@@ -46,14 +46,22 @@ class ProjectionError(MongoDBQueriesManagerBaseError):
     """ Raised when projection json is invalid. """
 
 
+class LogicalPopulationError(MongoDBQueriesManagerBaseError):
+    """ Raised when method fail to find logical population item. """
+
+
+class LogicalSubPopulationError(MongoDBQueriesManagerBaseError):
+    """ Raised when method fail to find logical sub population item. """
+
+
 class MongoDBQueriesManager:
     """ MongoDBQueriesManager class.
 
-    This class contain all method to convert string query to MongoDB query
+    This class contain all method to convert string query to MongoDB query.
 
     Attributes:
-        mongodb_operator (Dict[str, str]): MongoDB operator, used to convert query operators into MongoDB operators.
-        regex_dict (Dict[Union[str, re.Pattern], Any]): Contain all typing for cast into right format.
+        mongodb_operator (Dict[str, str]): MongoDB operator, used to convert query operators into MongoDB operators
+        regex_dict (Dict[Union[str, re.Pattern], Any]): Contain all typing for cast into right format
     """
 
     mongodb_operator: Dict[str, str] = {
@@ -73,7 +81,7 @@ class MongoDBQueriesManager:
         re.compile(r'^[12]\d{3}(-(0[1-9]|1[0-2])(-(0[1-9]|[12][0-9]|3[01]))?)(T|'
                    r' )?(([01][0-9]|2[0-3]):[0-5]\d(:[0-5]\d(\.\d+)?)?(Z|[+-]\d{2}:\d{2})?)?$'):
             lambda date: parse(date, languages=['fr', 'en']),
-        re.compile(r'^\w+(?=(,?,))(?:\1\w+)+$'): lambda list_value: list_value.split(','),
+        re.compile(r'^[A-Za-z ]+(?=(,?,))(?:\1[A-Za-z ]+)+$'): lambda list_value: list_value.split(','),
         re.compile(r'\/((?![*+?])(?:[^\r\n\[/\\]|\\.|\[(?:[^\r\n\]\\]|\\.)*\])+)'
                    r'\/((?:g(?:im?|mi?)?|i(?:gm?|mg?)?|m(?:gi?|ig?)?)?)'): re.compile,
         "true": lambda boolean: True,
@@ -88,7 +96,7 @@ class MongoDBQueriesManager:
         self.custom_cast_dict = casters
 
     def filter_logic(self, filter_params: str) -> Dict[str, Any]:
-        """ Build filter
+        """ Build filter.
 
         Args:
             filter_params (str): Filter params from url query (ie, 'name=John')
@@ -133,7 +141,7 @@ class MongoDBQueriesManager:
         return {key: {self.mongodb_operator[operator]: value}}
 
     def cast_value_logic(self, value: str) -> Any:
-        """ Cast value into right type
+        """ Cast value into right type.
 
         Args:
             value (str): Value to cast
@@ -162,10 +170,10 @@ class MongoDBQueriesManager:
 
     @staticmethod
     def find_operator(filter_params: str) -> str:
-        """Return the right operator
+        """Return the right operator.
 
         Args:
-            filter_params (str): Filter params (ie, 'name=John'):
+            filter_params (str): Filter params (ie, 'name=John')
 
         Returns:
             str: Return operator
@@ -177,10 +185,10 @@ class MongoDBQueriesManager:
 
     @staticmethod
     def sort_logic(sort_params: str) -> Optional[List[Tuple]]:
-        """ Convert sort query into MongoDB format
+        """ Convert sort query into MongoDB format.
 
         Notes:
-            Work if `sort=` into url query
+            Work if `sort=` into url query.
 
         Args:
             sort_params (str): Sort param from url query (ie, 'sort=-created_at')
@@ -204,7 +212,7 @@ class MongoDBQueriesManager:
 
     @staticmethod
     def text_operator_logic(text_param: str) -> str:
-        """ Convert text query into MongoDB format
+        """ Convert text query into MongoDB format.
 
         Args:
             text_param (str): Text param from url query (ie, '$text=java shop -coffee')
@@ -219,7 +227,7 @@ class MongoDBQueriesManager:
 
     @staticmethod
     def limit_logic(limit_param: str) -> int:
-        """ Convert limit query into MongoDB format
+        """ Convert limit query into MongoDB format.
 
         Args:
             limit_param (str): Limit param from url query (ie, 'limit=5')
@@ -242,7 +250,7 @@ class MongoDBQueriesManager:
 
     @staticmethod
     def skip_logic(skip_param: str) -> int:
-        """ Convert skip query into MongoDB format
+        """ Convert skip query into MongoDB format.
 
         Args:
             skip_param (str): Skip param from url query (ie, 'limit=5')
@@ -263,16 +271,47 @@ class MongoDBQueriesManager:
 
         return skip_value
 
-    @staticmethod
-    def projection_logic(projection_param: str, population: List[Dict[str, Any]]) -> Optional[Dict[str, Any]]:
-        """ Convert projection query into MongoDB format
+    @classmethod
+    def _iter_on_population(cls, population: List[Dict[str, Any]], field: str, sign: int) -> bool:
+        """ Used to add projection into population dict.
+
+        Args:
+            population (List[Dict[str, Any]]): List of asked population
+            field (str): Field value in string format
+            sign (int): The sign on the projection (0 for - / 1 for + or nothing)
+
+        Returns:
+            bool: Return true if the field as been added into a population otherwise false
+        """
+        pop_field = field.split('.', 1)
+        for current_population in population:
+            if current_population['path'] == pop_field[0]:
+                if pop_field[1].find('.') >= 0:
+                    if 'population' not in current_population:
+                        if not current_population.get('projection'):
+                            current_population['projection'] = {}
+                        current_population['projection'][pop_field[1]] = sign
+                    else:
+                        cls._iter_on_population(population=current_population['population'],
+                                                field=pop_field[1], sign=sign)
+                else:
+                    if not current_population.get('projection'):
+                        current_population['projection'] = {}
+                    current_population['projection'][pop_field[1]] = sign
+                return True
+        return False
+
+    @classmethod
+    def projection_logic(cls, projection_param: str,
+                         population: Optional[List[Dict[str, Any]]]) -> Optional[Dict[str, Any]]:
+        """ Convert projection query into MongoDB format.
 
         Notes:
-            Work if `projection=` into url query
+            Work if `projection=` into url query.
 
         Args:
             projection_param (str): Projection param from url query (ie, 'fields=id,url')
-            population (List[Dict[str, Any]]): Population query values
+            population (Optional[List[Dict[str, Any]]]): Optional population query values
 
         Returns:
             Optional[Dict[str, Any]]: Optional dictionary with MongoDB projection values
@@ -284,17 +323,9 @@ class MongoDBQueriesManager:
             return None
 
         for param in value.split(','):
-            if param.find('.') > 0:
-                pop_field = param[1:].split('.', 1) if param.startswith('-') else param.split('.', 1)
-                found = False
-                for pop in population:
-                    if pop['path'] == pop_field[0]:
-                        if not pop.get('projection'):
-                            pop['projection'] = {}
-                        pop['projection'][pop_field[1]] = 0 if param.startswith('-') else 1
-                        found = True
-                        break
-                if found:
+            if population and param.find('.') > 0:
+                if cls._iter_on_population(population=population, field=param[1:] if param.startswith('-') else param,
+                                           sign=0 if param.startswith('-') else 1):
                     continue
 
             if param.startswith('-'):
@@ -307,4 +338,73 @@ class MongoDBQueriesManager:
                     raise ProjectionError('Fail to decode projection') from err
             else:
                 projection_params_final[param] = 1
-        return projection_params_final
+        return projection_params_final if projection_params_final != {} else None
+
+    @classmethod
+    def format_populate_value(cls, mongodb_query: Dict[str, Any], population_value: str) -> None:
+        """ Convert population query into dict format.
+
+        Args:
+            mongodb_query (Dict[str, Any]): The actual mongodb query
+            population_value (str): Population string value
+
+        Returns:
+            None: No return.
+        """
+        if population_value.find('.') >= 0:
+            cls._iter_on_populate_value(mongodb_query, population_value)
+        else:
+            mongodb_query['population'].append({'path': population_value, 'projection': None})
+
+    @classmethod
+    def _iter_on_populate_value(cls, mongodb_query: Dict[str, Any], population_value: str) -> None:
+        """ Used into format_populate_value method (split to respect the pylint config)
+
+        Args:
+            mongodb_query (Dict[str, Any]): The actual mongodb query
+            population_value (str): Population string value
+
+        Returns:
+            None: No return.
+        """
+        path, sub_path = population_value.split('.', 1)
+        if 'population' in mongodb_query:
+            for population in mongodb_query['population']:
+                if path == population['path']:
+                    if 'population' not in population:
+                        population['population'] = []
+
+                    if sub_path.find('.') >= 0:
+                        cls._iter_on_sub_populate_value(population=population, sub_path=sub_path)
+                    else:
+                        population['population'].append({'path': sub_path, 'projection': None})
+                    break
+            else:
+                raise LogicalPopulationError('Fail to find logical population item')
+        else:
+            raise LogicalPopulationError('Fail to find logical population item')
+
+    @classmethod
+    def _iter_on_sub_populate_value(cls, population: Dict[str, Any], sub_path: str) -> None:
+        """ Used into _iter_on_populate_value method (split to respect the pylint config)
+
+        Args:
+            population (Dict[str, Any]): The actual population
+            sub_path (str): Sub path value
+
+        Returns:
+            None: No return.
+        """
+        path, sub_path = sub_path.split('.', 1)
+        for sub_population in population['population']:
+            if path == sub_population['path']:
+                if sub_path.find('.') >= 0:
+                    cls.format_populate_value(mongodb_query=sub_population,
+                                              population_value=sub_path)
+                else:
+                    if 'population' not in sub_population:
+                        sub_population['population'] = []
+                    sub_population['population'].append({'path': sub_path, 'projection': None})
+                break
+        else:
+            raise LogicalSubPopulationError('Fail to find logical sub population item')
